@@ -5,15 +5,13 @@
 # A Linux console text editor for the masses.
 # :title: Diakonos
 #
-# Author:: Pistos (irc.freenode.net)
-# http://diakonos.pist0s.ca/
-# Copyright (c) 2004-2011 Pistos
+# Author:: Pistos
+# Copyright (c) 2004-2023 Pistos
 #
-# This software is released under the MIT licence.
+# This software is released under the GNU GPL licence v3.
 # See the LICENCE file included with this program, or
 # http://www.opensource.org/licenses/mit-license.php
 #
-require 'curses'
 require 'open3'
 require 'thread'
 require 'English'
@@ -21,15 +19,15 @@ require 'set'
 require 'yaml'
 require 'digest/md5'
 require 'timeout'
+require 'fileutils'
 
 require 'diakonos/core-ext/object'
 require 'diakonos/core-ext/enumerable'
 require 'diakonos/core-ext/regexp'
 require 'diakonos/core-ext/hash'
 require 'diakonos/core-ext/string'
-require 'diakonos/core-ext/fixnum'
-require 'diakonos/core-ext/bignum'
 require 'diakonos/buffer-hash'
+require 'diakonos/number-fitter'
 require 'diakonos/sized-array'
 
 require 'diakonos/version'
@@ -38,6 +36,7 @@ require 'diakonos/installation'
 require 'diakonos/about'
 require 'diakonos/buffer-management'
 require 'diakonos/config'
+require 'diakonos/config-file'
 require 'diakonos/cursor'
 require 'diakonos/functions'
 require 'diakonos/functions/basics'
@@ -56,14 +55,14 @@ require 'diakonos/functions/tags'
 require 'diakonos/functions/text-manipulation'
 require 'diakonos/functions-deprecated'
 require 'diakonos/help'
-require 'diakonos/display'
-require 'diakonos/display/format'
+require 'diakonos/interaction-handler'
 require 'diakonos/grep'
 require 'diakonos/hooks'
 require 'diakonos/interaction'
 require 'diakonos/keying'
 require 'diakonos/logging'
 require 'diakonos/list'
+require 'diakonos/mouse'
 require 'diakonos/search'
 require 'diakonos/sessions'
 
@@ -84,11 +83,13 @@ require 'diakonos/buffer/undo'
 require 'diakonos/clipboard'
 require 'diakonos/clipboard-klipper'
 require 'diakonos/clipboard-klipper-dbus'
+require 'diakonos/clipboard-osx'
 require 'diakonos/clipboard-xclip'
 require 'diakonos/extension'
 require 'diakonos/extension-set'
 require 'diakonos/key-map'
 require 'diakonos/mode'
+require 'diakonos/range'
 require 'diakonos/readline'
 require 'diakonos/readline/functions'
 
@@ -121,12 +122,35 @@ module Diakonos
 
   LANG_TEXT = 'text'
 
+  # @return [Array] the filename and line number parsed
+  def self.parse_filename_and_line_number( s )
+    if(
+      # Ruby
+      s =~ /from (.+):(\d+)/ ||
+      # Python
+      s =~ /File "(.+)", line (\d+)/ ||
+      # Perl
+      s =~ /at (.+) line (\d+)/ ||
+      # generic
+      s =~ /^(.+):(\d+)/
+    )
+      [ $1, ( $2.to_i - 1 ) ]
+    else
+      [ s, nil ]
+    end
+  end
+
   class Diakonos
 
     attr_reader :diakonos_home, :script_dir, :clipboard,
       :list_filename, :hooks, :indenters, :indenters_next_line, :unindenters, :closers,
       :functions_last, :there_was_non_movement, :do_display, :testing, :buffers
 
+    # Temporarily a writer, so that collaborating classes can get at it
+    # TODO: Rewrite things so that this doesn't need to be a writer.
+    # e.g. pass do_display to methods as needed.
+    attr_writer :do_display
+
     include ::Diakonos::Functions
 
     def initialize( argv = [] )
@@ -142,6 +166,12 @@ module Diakonos
       @config_filename = nil
       parse_options argv
 
+      # These requires are down here instead of up with the others so that
+      # uninstall can be done without needing the curses lib (which may not
+      # be installed).
+      require 'diakonos/display'
+      require 'diakonos/display/format'
+
       init_help
 
       @debug          = File.new( File.join( @diakonos_home, 'debug.log' ), 'w' )
@@ -183,12 +213,10 @@ module Diakonos
       @display_queue_mutex    = Mutex.new
       @display_queue          = nil
       @do_display             = true
-      @iline_mutex            = Mutex.new
+      @context_line_mutex     = Mutex.new
 
       @tag_stack              = Array.new
       @last_search_regexps    = nil
-      @iterated_choice        = nil
-      @choice_iterations      = 0
       @there_was_non_movement = false
       @status_vars            = Hash.new
 
@@ -209,7 +237,7 @@ module Diakonos
     end
 
     def mkdir( dir )
-      if ! FileTest.exists?( dir )
+      if ! FileTest.exist?( dir )
         Dir.mkdir dir
       end
     end
@@ -240,7 +268,7 @@ module Diakonos
           regexp = argv.shift
           files = `egrep -rl '#{regexp}' *`.split( /\n/ )
           if files.any?
-            @files.concat( files.map { |f| session_file_hash_for f } )
+            @files.concat( files.map { |f| Session.file_hash_for f } )
             script = "\nfind '#{regexp}', case_sensitive: true"
             @post_load_script << script
           end
@@ -250,7 +278,7 @@ module Diakonos
             print_usage
             exit 1
           else
-            h = session_file_hash_for( filename )
+            h = Session.file_hash_for( filename )
             h[ 'read_only' ] = true
             @read_only_files.push h
           end
@@ -267,7 +295,7 @@ module Diakonos
           exit 0
         else
           # a name of a file to open
-          @files.push session_file_hash_for( arg )
+          @files.push Session.file_hash_for( arg )
         end
       end
     end
@@ -289,86 +317,79 @@ module Diakonos
 
     # -----------------------------------------------------------------------
 
+    def create_buffers_from_files
+      @files.each do |file_info|
+        @buffers << Buffer.new(file_info)
+      end
+    end
+
     def start
       require 'diakonos/window'
 
-      initialize_display
-
-      @files.each do |file|
-        @buffers << Buffer.new( file )
-      end
+      create_buffers_from_files
       @files = []
       @read_only_files.each do |file|
         @buffers << Buffer.new( file )
       end
       if ! @testing
-        session_buffers = session_startup
-      end
-      session_buffer_number = @session[ 'buffer_current' ] || 1
-      @files.each do |file_info|
-        @buffers << Buffer.new( file_info )
+        session_startup
       end
+      create_buffers_from_files
+      @files = []
       if @buffers.empty?
         @buffers << Buffer.new
       end
 
+      initialize_display
       @buffers.each do |buffer|
         buffer.reset_display
       end
 
-      if ENV[ 'COLORTERM' ] == 'gnome-terminal'
-        help_key = 'Shift-F1'
-      else
-        help_key = 'F1'
-      end
-      set_iline "Diakonos #{VERSION} (#{LAST_MODIFIED})   #{help_key} for help  F12 to configure  Ctrl-Q to quit"
+      set_initial_iline
 
-      scripts = @extensions.scripts + Dir[ "#{@script_dir}/*" ]
-      scripts.each do |script|
-        begin
-          require script
-        rescue Exception => e
-          show_exception(
-            e,
-            [
-              "There is a syntax error in the script.",
-              "An invalid hook name was used."
-            ]
-          )
-        end
-      end
+      run_scripts
 
       @hooks.each do |hook_name, hook|
         hook.sort { |a,b| a[ :priority ] <=> b[ :priority ] }
       end
 
-      if session_buffers
-        session_buffers.each do |buffer|
-          close_buffer buffer
-        end
-      end
+      handle_stale_session_files
 
-      if @post_load_script
-        eval @post_load_script
-      end
+      create_buffers_from_files
 
-      run_hook_procs :after_startup
+      session_buffer_number = @session.buffer_current || 1
       if ! switch_to_buffer_number( session_buffer_number )
         debug_log "Failed to switch to buffer #{session_buffer_number.inspect}"
         switch_to_buffer_number 1
       end
 
+      run_hook_procs :after_startup
+      if @post_load_script
+        begin
+          eval @post_load_script
+        rescue Exception => e
+          show_exception(
+            e,
+            [ "There is an error in the post-load script:\n#{@post_load_script}" ]
+          )
+        end
+      end
+
       @buffers.each do |b|
         run_hook_procs :after_open, b
         b.cursor_to( b.last_row, b.last_col, Buffer::DONT_DISPLAY )
       end
       buffer_current.cursor_to( buffer_current.last_row, buffer_current.last_col, Buffer::DONT_DISPLAY )
 
-      if @config_problems.any?
+      if @configs.any? { |c| c.problems.any? }
         File.open( @error_filename, "w" ) do |f|
-          f.puts "There are #{@config_problems.size} problems with the configuration file(s):"
-          @config_problems.each do |p|
-            f.puts p
+          f.puts "There are problems with the configuration file(s):"
+          @configs.each do |c|
+            next  if c.problems.empty?
+
+            f.puts
+            f.print "#{c}:\n\t"
+            f.puts c.problems.join("\n\t")
           end
         end
         open_file @error_filename
@@ -399,6 +420,32 @@ module Diakonos
       @debug.close
     end
 
+    def set_initial_iline
+      if ENV['COLORTERM'] == 'gnome-terminal'
+        help_key = 'Shift-F1'
+      else
+        help_key = 'F1'
+      end
+      set_iline "Diakonos #{VERSION} (#{LAST_MODIFIED})   #{help_key} for help  F12 to configure  Ctrl-Q to quit"
+    end
+
+    def run_scripts
+      scripts = @extensions.scripts + Dir[ "#{@script_dir}/*" ]
+      scripts.each do |script|
+        begin
+          require script
+        rescue Exception => e
+          show_exception(
+            e,
+            [
+              "There is a syntax error in the script.",
+              "An invalid hook name was used."
+            ]
+          )
+        end
+      end
+    end
+
     def uninstall( confirm = true )
       inst = ::Diakonos::INSTALL_SETTINGS[ :installed ]
 
@@ -418,7 +465,6 @@ module Diakonos
         end
       end
 
-      require 'fileutils'
       inst[ :files ].each do |f|
         FileUtils.rm f
       end
@@ -427,7 +473,7 @@ module Diakonos
           FileUtils.rmdir d
         rescue Errno::ENOTEMPTY
         end
-        if File.exists? d
+        if File.exist? d
           $stderr.puts "(#{d} not removed)"
         end
       end
@@ -504,7 +550,7 @@ module Diakonos
         tagfile = "./tags"
       end
 
-      if ! FileTest.exists? tagfile
+      if ! FileTest.exist? tagfile
         set_iline "(tags file not found)"
       else
         IO.foreach( tagfile ) do |line_|
@@ -534,24 +580,6 @@ module Diakonos
       temp
     end
 
-    # @return [Array] the filename and line number parsed
-    def parse_filename_and_line_number( s )
-      if(
-        # Ruby
-        s =~ /from (.+):(\d+)/ ||
-        # Python
-        s =~ /File "(.+)", line (\d+)/ ||
-        # Perl
-        s =~ /at (.+) line (\d+)/ ||
-        # generic
-        s =~ /^(.+):(\d+)/
-      )
-        [ $1, ( $2.to_i - 1 ) ]
-      else
-        [ s, nil ]
-      end
-    end
-
   end
 
 end
