@@ -5,10 +5,7 @@ module Diakonos
       File.open( @about_filename, "w" ) do |f|
         inst = ::Diakonos::INSTALL_SETTINGS
 
-        configs = @configs.map { |h|
-          source = h[:source] ? "(included by #{h[:source]})" : ''
-          "#{h[:filename]}\t#{source}"
-        }.join("\n")
+        configs = @configs.map(&:to_s).join("\n")
 
         ext_loaded = @extensions.loaded_extensions.sort_by { |e|
           e.name.downcase
@@ -23,21 +20,30 @@ module Diakonos
           "### #{e} (NOT LOADED)"
         }.join( "\n" )
 
+        installation_artifact = File.join(inst[:lib_dir], 'diakonos', 'installation.rb')
+        if File.exist?(installation_artifact)
+          install_time = File.mtime(installation_artifact)
+        else
+          install_time = "--"
+        end
+
         f.puts %{
 # About Diakonos
 
-Licence:        MIT Licence
+Licence:        GNU GPL v3
 Copyright:      Copyright (c) 2004-#{ Time.now.year } Pistos
 
 ## Version
 
 Version:        #{ ::Diakonos::VERSION }
 Code Date:      #{ ::Diakonos::LAST_MODIFIED }
-Install Time:   #{ File.mtime( File.join( inst[ :lib_dir ], 'diakonos', 'installation.rb' ) ) }
+Install Time:   #{ install_time }
+Ruby Version:   #{ ::RUBY_VERSION }
 
 ## Paths
 
 Home dir:       #{ @diakonos_home }
+Session dir:    #{ @session.dir }
 
 ### Installation
 
@@ -61,4 +67,4 @@ System library dir: #{ inst[ :lib_dir ] }
     end
 
   end
-end
\ No newline at end of file
+end
