@@ -1,9 +1,10 @@
 module Diakonos
 
   class Buffer
-    attr_reader :name, :original_language, :changing_selection, :read_only,
+    attr_reader :name, :original_language, :language, :changing_selection, :read_only,
       :tab_size, :selection_mode
     attr_writer :desired_column, :read_only
+    attr_accessor :lines
 
     TYPING                 = true
     STOPPED_TYPING         = true
@@ -54,9 +55,9 @@ module Diakonos
         @lines[ 0 ] = ""
       else
         @name = File.expand_path( @name )
-        if FileTest.exists? @name
+        if FileTest.exist? @name
           @lines = IO.readlines( @name )
-          if ( @lines.length == 0 ) or ( @lines[ -1 ][ -1..-1 ] == "\n" )
+          if ( @lines.length == 0 ) || ( @lines[ -1 ][ -1..-1 ] == "\n" )
             @lines.push ""
           end
           @lines = @lines.collect do |line|
@@ -98,11 +99,11 @@ module Diakonos
         tabs_subbed = false
         @lines.collect! do |line|
           new_line = line.expand_tabs( @tab_size )
-          tabs_subbed = ( tabs_subbed or new_line != line )
+          tabs_subbed = ( tabs_subbed || new_line != line )
           # Return value for collect:
           new_line
         end
-        @modified = ( @modified or tabs_subbed )
+        @modified = ( @modified || tabs_subbed )
         if tabs_subbed
           $diakonos.set_iline "(spaces substituted for tab characters)"
         end
@@ -114,8 +115,8 @@ module Diakonos
 
     def configure(
       language = (
-        $diakonos.get_language_from_shabang( @lines[ 0 ] ) or
-        $diakonos.get_language_from_name( @name ) or
+        $diakonos.get_language_from_shabang( @lines[ 0 ] ) ||
+        $diakonos.get_language_from_name( @name ) ||
         LANG_TEXT
       )
     )
@@ -305,7 +306,7 @@ module Diakonos
     def columnize( delimiter = /=>?|:|,/, num_spaces_padding = 1 )
       take_snapshot
 
-      lines = selected_lines
+      lines = self.selected_lines
       column_width = 0
       lines.each do |line|
         pos = ( line =~ delimiter )
@@ -343,14 +344,20 @@ module Diakonos
 
     def comment_out
       take_snapshot
+
       one_modified = false
-      selected_lines.each do |line|
+      closer = @settings[ "lang.#{@language}.comment_close_string" ].to_s
+
+      self.selected_lines.each do |line|
         next  if line.strip.empty?
         old_line = line.dup
         line.gsub!( /^(\s*)/, "\\1" + @settings[ "lang.#{@language}.comment_string" ].to_s )
-        line << @settings[ "lang.#{@language}.comment_close_string" ].to_s
+        if ! closer.empty? && line !~ /#{Regexp.escape(closer)}$/
+          line << closer
+        end
         one_modified ||= ( line != old_line )
       end
+
       if one_modified
         set_modified
       end
@@ -361,10 +368,13 @@ module Diakonos
       comment_string = Regexp.escape( @settings[ "lang.#{@language}.comment_string" ].to_s )
       comment_close_string = Regexp.escape( @settings[ "lang.#{@language}.comment_close_string" ].to_s )
       one_modified = false
-      selected_lines.each do |line|
+      self.selected_lines.each do |line|
         old_line = line.dup
-        line.gsub!( /^(\s*)#{comment_string}/, "\\1" )
-        line.gsub!( /#{comment_close_string}$/, '' )
+        comment_regexp = /^(\s*)#{comment_string}/
+        line.gsub!( comment_regexp, "\\1" )
+        if line !~ comment_regexp
+          line.gsub!( /#{comment_close_string}$/, '' )
+        end
         one_modified ||= ( line != old_line )
       end
       if one_modified
@@ -381,7 +391,9 @@ module Diakonos
         [ @lines[ row ][ col..-1 ] ] +
         @lines[ (row+1)..-1 ]
       cursor_to( row + 1, 0 )
-      parsed_indent  if @auto_indent
+      if @auto_indent
+        parsed_indent  undoable: false
+      end
       set_modified
     end
 
@@ -417,7 +429,7 @@ module Diakonos
     def row_to_y( row )
       return nil if row.nil?
       y = row - @top_line
-      y = nil if ( y < 0 ) or ( y > @top_line + $diakonos.main_window_height - 1 )
+      y = nil if ( y < 0 ) || ( y > @top_line + $diakonos.main_window_height - 1 )
       y
     end
 
@@ -425,7 +437,7 @@ module Diakonos
     def column_to_x( col )
       return nil if col.nil?
       x = col - @left_column
-      x = nil if ( x < 0 ) or ( x > @left_column + Curses::cols - 1 )
+      x = nil if ( x < 0 ) || ( x > @left_column + Curses::cols - 1 )
       x
     end
 
@@ -471,22 +483,42 @@ module Diakonos
 
       height = [ $diakonos.main_window_height, @lines.length ].min
 
-      @last_row = @last_row.fit( @top_line, @top_line + height - 1 )
-      if @last_row - @top_line < @settings[ "view.margin.y" ]
-        @last_row = @top_line + @settings[ "view.margin.y" ]
-        @last_row = @last_row.fit( @top_line, @top_line + height - 1 )
+      @last_row = NumberFitter.fit(
+        number: @last_row,
+        min: @top_line,
+        max: @top_line + height - 1,
+      )
+
+      if @last_row - @top_line < @settings["view.margin.y"]
+        @last_row = NumberFitter.fit(
+          number: @top_line + @settings["view.margin.y"],
+          min: @top_line,
+          max: @top_line + height - 1,
+        )
       elsif @top_line + height - 1 - @last_row < @settings[ "view.margin.y" ]
-        @last_row = @top_line + height - 1 - @settings[ "view.margin.y" ]
-        @last_row = @last_row.fit( @top_line, @top_line + height - 1 )
+        @last_row = NumberFitter.fit(
+          number: @top_line + height - 1 - @settings[ "view.margin.y" ],
+          min: @top_line,
+          max: @top_line + height - 1,
+        )
       end
-      @last_col = @last_col.fit( @left_column, [ @left_column + Curses::cols - 1, @lines[ @last_row ].length ].min )
+
+      @last_col = NumberFitter.fit(
+        number: @last_col,
+        min: @left_column,
+        max: [
+          @left_column + Curses::cols - 1,
+          @lines[@last_row].length
+        ].min
+      )
+
       @last_screen_y = @last_row - @top_line
       @last_screen_x = tab_expanded_column( @last_col, @last_row ) - @left_column
 
       record_mark_start_and_end
 
       if changed != 0
-        if ! @changing_selection && selecting?
+        if ! @auto_anchored && ! @changing_selection && selecting?
           remove_selection DONT_DISPLAY
         end
 
@@ -555,7 +587,7 @@ module Diakonos
         line = @lines[ row ]
         if ! line.strip.empty? && ( line !~ @settings[ "lang.#{@language}.context.ignore" ] )
           level = indentation_level( row )
-          if level < clevel and level > -1
+          if level < clevel && level > -1
             retval.unshift line
             clevel = level
             break  if clevel == 0
