@@ -9,7 +9,11 @@ module Diakonos
       old_last_row = @last_row
       old_last_col = @last_col
 
-      row = row.fit( 0, @lines.length - 1 )
+      row = NumberFitter.fit(
+        number: row,
+        min: 0,
+        max: @lines.length - 1,
+      )
 
       if col < 0
         if adjust_row
@@ -56,11 +60,12 @@ module Diakonos
 
       record_mark_start_and_end
 
-      removed = false
-      if not @changing_selection and selecting?
+      selection_removed = false
+      if ! @auto_anchored && ! @changing_selection && selecting?
         remove_selection( DONT_DISPLAY )
-        removed = true
+        selection_removed = true
       end
+      @auto_anchored = false
 
       old_pair = @text_marks[ :pair ]
       if @settings[ 'view.pairs.highlight' ]
@@ -70,7 +75,7 @@ module Diakonos
       end
       highlight_changed = old_pair != @text_marks[ :pair ]
 
-      if removed || ( do_display && ( selecting? || view_changed || highlight_changed ) )
+      if selection_removed || ( do_display && ( selecting? || view_changed || highlight_changed ) )
         display
       else
         $diakonos.display_mutex.synchronize do
@@ -93,7 +98,7 @@ module Diakonos
       when BOL_ZERO
         col = 0
       when BOL_FIRST_CHAR
-        col = ( ( @lines[ row ] =~ /\S/ ) or 0 )
+        col = ( ( @lines[ row ] =~ /\S/ ) || 0 )
       when BOL_ALT_ZERO
         if @last_col == 0
           col = ( @lines[ row ] =~ /\S/ )
@@ -102,7 +107,7 @@ module Diakonos
         end
         #when BOL_ALT_FIRST_CHAR
       else
-        first_char_col = ( ( @lines[ row ] =~ /\S/ ) or 0 )
+        first_char_col = ( ( @lines[ row ] =~ /\S/ ) || 0 )
         if @last_col == first_char_col
           col = 0
         else
@@ -170,7 +175,7 @@ module Diakonos
         break if( pan_view( amount, DONT_DISPLAY ) != amount )
       end
 
-      @top_line != old_top_line or @left_column != old_left_column
+      @top_line != old_top_line || @left_column != old_left_column
     end
 
     def go_to_line( line = nil, column = nil, do_display = DO_DISPLAY )
@@ -327,4 +332,4 @@ module Diakonos
 
   end
 
-end
\ No newline at end of file
+end
