@@ -2,24 +2,24 @@ module Diakonos
 
 class CTag
     attr_reader :file, :command, :kind, :rest
-    
+
     def initialize( file, command, kind, rest )
         @file = file
         @command = command
         @kind = kind
         @rest = rest
     end
-    
+
     def to_s
         "#{@file}:#{@command} (#{@kind}) #{@rest}"
     end
-    
+
     def == ( other )
         (
-            other and
-            @file == other.file and
-            @command == other.command and
-            @kind == other.kind and
+            other &&
+            @file == other.file &&
+            @command == other.command &&
+            @kind == other.kind &&
             @rest == other.rest
         )
     end
