@@ -1,3 +1,5 @@
+require 'curses'
+
 module Diakonos
   DO_REDRAW             = true
   DONT_REDRAW           = false
@@ -25,17 +27,24 @@ module Diakonos
         Curses::nonl
         Curses::raw
         Curses::noecho
+        if @settings['mouse']
+          Curses::mousemask(Curses::ALL_MOUSE_EVENTS)
+        end
 
         if Curses::has_colors?
           Curses::start_color
-          Curses::init_pair( Curses::COLOR_BLACK, Curses::COLOR_BLACK, Curses::COLOR_BLACK )
-          Curses::init_pair( Curses::COLOR_RED, Curses::COLOR_RED, Curses::COLOR_BLACK )
-          Curses::init_pair( Curses::COLOR_GREEN, Curses::COLOR_GREEN, Curses::COLOR_BLACK )
-          Curses::init_pair( Curses::COLOR_YELLOW, Curses::COLOR_YELLOW, Curses::COLOR_BLACK )
-          Curses::init_pair( Curses::COLOR_BLUE, Curses::COLOR_BLUE, Curses::COLOR_BLACK )
-          Curses::init_pair( Curses::COLOR_MAGENTA, Curses::COLOR_MAGENTA, Curses::COLOR_BLACK )
-          Curses::init_pair( Curses::COLOR_CYAN, Curses::COLOR_CYAN, Curses::COLOR_BLACK )
-          Curses::init_pair( Curses::COLOR_WHITE, Curses::COLOR_WHITE, Curses::COLOR_BLACK )
+          Curses::use_default_colors
+
+          # -1 means use the terminal's current/default background, which may even have some transparency
+          background_colour = settings['colour.background'] || -1
+          Curses::init_pair( Curses::COLOR_BLACK, Curses::COLOR_BLACK, background_colour )
+          Curses::init_pair( Curses::COLOR_RED, Curses::COLOR_RED, background_colour )
+          Curses::init_pair( Curses::COLOR_GREEN, Curses::COLOR_GREEN, background_colour )
+          Curses::init_pair( Curses::COLOR_YELLOW, Curses::COLOR_YELLOW, background_colour )
+          Curses::init_pair( Curses::COLOR_BLUE, Curses::COLOR_BLUE, background_colour )
+          Curses::init_pair( Curses::COLOR_MAGENTA, Curses::COLOR_MAGENTA, background_colour )
+          Curses::init_pair( Curses::COLOR_CYAN, Curses::COLOR_CYAN, background_colour )
+          Curses::init_pair( Curses::COLOR_WHITE, Curses::COLOR_WHITE, background_colour )
           @colour_pairs.each do |cp|
             Curses::init_pair( cp[ :number ], cp[ :fg ], cp[ :bg ] )
           end
@@ -53,8 +62,18 @@ module Diakonos
       @win_status.attrset @settings[ 'status.format' ]
       @win_interaction = ::Diakonos::Window.new( 1, Curses::cols, Curses::lines - 1, 0 )
 
-      if @settings[ 'context.visible' ]
-        if @settings[ 'context.combined' ]
+      @interaction_handler = InteractionHandler.new(
+        win_main: @win_main,
+        win_interaction: @win_interaction,
+        cursor_manager: self,
+        testing: @testing,
+        choice_delay: @settings['interaction.choice_delay'],
+        blink_string: @settings['interaction.blink_string'],
+        blink_duration: @settings['interaction.blink_duration']
+      )
+
+      if @settings['context.visible']
+        if @settings['context.combined']
           pos = 1
         else
           pos = 3
@@ -105,7 +124,7 @@ module Diakonos
       # One line for the input line
       # One line for the context line
       retval = Curses::lines - 2
-      if @settings[ "context.visible" ] and not @settings[ "context.combined" ]
+      if @settings['context.visible'] && ! @settings['context.combined']
         retval = retval - 1
       end
       retval
@@ -115,24 +134,12 @@ module Diakonos
       Curses::cols
     end
 
-    # Display text on the interaction line.
-    def set_iline( string = "" )
-      return  if @testing
-      return  if @readline
-
-      @iline = string
-      Curses::curs_set 0
-      @win_interaction.setpos( 0, 0 )
-      @win_interaction.addstr( "%-#{Curses::cols}s" % @iline )
-      @win_interaction.refresh
-      Curses::curs_set 1
-      string.length
+    def set_iline(s = "")
+      @interaction_handler.set_iline s
     end
 
-    def set_iline_if_empty( string )
-      if @iline.nil? || @iline.empty?
-        set_iline string
-      end
+    def set_iline_if_empty(s)
+      @interaction_handler.set_iline_if_empty s
     end
 
     def set_status_variable( identifier, value )
@@ -182,7 +189,7 @@ module Diakonos
             var_array.push ''
           end
         when 'session_name'
-          var_array.push @session[ 'name' ]
+          var_array.push @session.name
         when "type"
           var_array.push buffer_current.original_language
         when /^@/
@@ -242,7 +249,7 @@ module Diakonos
           truncation = [ @settings[ "context.max_levels" ], context.length ].min
           max_length = [
             ( Curses::cols / truncation ) - @settings[ "context.separator" ].length,
-            ( @settings[ "context.max_segment_width" ] or Curses::cols )
+            ( @settings[ "context.max_segment_width" ] || Curses::cols )
           ].min
           line = nil
           context_subset = context[ 0...truncation ]
@@ -260,7 +267,7 @@ module Diakonos
           end
         end
 
-        @iline_mutex.synchronize do
+        @context_line_mutex.synchronize do
           @win_context.attrset @settings[ "context.format" ]
           @win_context.addstr( " " * ( Curses::cols - chars_printed ) )
           @win_context.refresh
@@ -312,30 +319,6 @@ module Diakonos
 
     end
 
-    def show_message( message, non_interaction_duration = @settings[ 'interaction.choice_delay' ] )
-      terminate_message
-
-      @message_expiry = Time.now + non_interaction_duration
-      @message_thread = Thread.new do
-        time_left = @message_expiry - Time.now
-        while time_left > 0
-          set_iline "(#{time_left.round}) #{message}"
-          @win_main.setpos( @saved_main_y, @saved_main_x )
-          sleep 1
-          time_left = @message_expiry - Time.now
-        end
-        set_iline message
-        @win_main.setpos( @saved_main_y, @saved_main_x )
-      end
-    end
-
-    def terminate_message
-      if @message_thread and @message_thread.alive?
-        @message_thread.terminate
-        @message_thread = nil
-      end
-    end
-
     def refresh_all
       @win_main.refresh
       if @win_context
