@@ -29,7 +29,7 @@ module Diakonos
     end
 
     def insert_tab
-      type_character TAB
+      type_character "\t"
     end
 
     def parsed_indent
@@ -37,7 +37,7 @@ module Diakonos
         @do_display = false
         mark = buffer_current.selection_mark
         (mark.start_row..mark.end_row).each do |row|
-          buffer_current.parsed_indent row, Buffer::DONT_DISPLAY
+          buffer_current.parsed_indent  row: row, do_display: false
         end
         @do_display = true
         display_buffer buffer_current
@@ -68,4 +68,4 @@ module Diakonos
     end
 
   end
-end
\ No newline at end of file
+end
