@@ -2,13 +2,15 @@
 # This copy is here to permit the tests to execute.
 
 module Diakonos
+  root = File.expand_path("../..", __dir__)
+
   INSTALL_SETTINGS = {
-    :prefix   => '.',
-    :bin_dir  => 'bin',
-    :doc_dir  => '.',
-    :help_dir => 'help',
-    :conf_dir => '.',
-    :lib_dir  => 'lib',
+    :prefix   => root,
+    :bin_dir  => File.join(root, "bin"),
+    :doc_dir  => root,
+    :help_dir => File.join(root, "help"),
+    :conf_dir => root,
+    :lib_dir  => File.join(root, "lib"),
     :installed => {
       :files => [],
       :dirs => [],
