@@ -2,50 +2,41 @@ module Diakonos
   module Keying
 
     KEYSTRINGS = [
-        "ctrl+space",   # 0
-        "ctrl+a",       # 1
-        "ctrl+b",       # 2
-        "ctrl+c",       # 3
-        "ctrl+d",       # 4
-        "ctrl+e",       # 5
-        "ctrl+f",       # 6
-        "ctrl+g",       # 7
-        nil,            # 8
-        "tab",          # 9
-        "ctrl+j",       # 10
-        "ctrl+k",       # 11
-        "ctrl+l",       # 12
-        "enter",        # 13
-        "ctrl+n",       # 14
-        "ctrl+o",       # 15
-        "ctrl+p",       # 16
-        "ctrl+q",       # 17
-        "ctrl+r",       # 18
-        "ctrl+s",       # 19
-        "ctrl+t",       # 20
-        "ctrl+u",       # 21
-        "ctrl+v",       # 22
-        "ctrl+w",       # 23
-        "ctrl+x",       # 24
-        "ctrl+y",       # 25
-        "ctrl+z",       # 26
-        "esc",          # 27
-        nil,            # 28
-        nil,            # 29
-        nil,            # 30
-        nil,            # 31
-        "space",        # 32
-        33.chr, 34.chr, 35.chr, 36.chr, 37.chr, 38.chr, 39.chr,
-        40.chr, 41.chr, 42.chr, 43.chr, 44.chr, 45.chr, 46.chr, 47.chr, 48.chr, 49.chr,
-        50.chr, 51.chr, 52.chr, 53.chr, 54.chr, 55.chr, 56.chr, 57.chr, 58.chr, 59.chr,
-        60.chr, 61.chr, 62.chr, 63.chr, 64.chr, 65.chr, 66.chr, 67.chr, 68.chr, 69.chr,
-        70.chr, 71.chr, 72.chr, 73.chr, 74.chr, 75.chr, 76.chr, 77.chr, 78.chr, 79.chr,
-        80.chr, 81.chr, 82.chr, 83.chr, 84.chr, 85.chr, 86.chr, 87.chr, 88.chr, 89.chr,
-        90.chr, 91.chr, 92.chr, 93.chr, 94.chr, 95.chr, 96.chr, 97.chr, 98.chr, 99.chr,
-        100.chr, 101.chr, 102.chr, 103.chr, 104.chr, 105.chr, 106.chr, 107.chr, 108.chr, 109.chr,
-        110.chr, 111.chr, 112.chr, 113.chr, 114.chr, 115.chr, 116.chr, 117.chr, 118.chr, 119.chr,
-        120.chr, 121.chr, 122.chr, 123.chr, 124.chr, 125.chr, 126.chr,
-        "backspace"    # 127
+      "ctrl+space",   # 0
+      "ctrl+a",       # 1
+      "ctrl+b",       # 2
+      "ctrl+c",       # 3
+      "ctrl+d",       # 4
+      "ctrl+e",       # 5
+      "ctrl+f",       # 6
+      "ctrl+g",       # 7
+      nil,            # 8
+      "tab",          # 9
+      "ctrl+j",       # 10
+      "ctrl+k",       # 11
+      "ctrl+l",       # 12
+      "enter",        # 13
+      "ctrl+n",       # 14
+      "ctrl+o",       # 15
+      "ctrl+p",       # 16
+      "ctrl+q",       # 17
+      "ctrl+r",       # 18
+      "ctrl+s",       # 19
+      "ctrl+t",       # 20
+      "ctrl+u",       # 21
+      "ctrl+v",       # 22
+      "ctrl+w",       # 23
+      "ctrl+x",       # 24
+      "ctrl+y",       # 25
+      "ctrl+z",       # 26
+      "esc",          # 27
+      nil,            # 28
+      nil,            # 29
+      nil,            # 30
+      nil,            # 31
+      "space",        # 32
+    ] + (33..126).map(&:chr) + [
+      "backspace"    # 127
     ]
 
     def self.keycodes_for( str )
@@ -166,11 +157,18 @@ module Diakonos
           "resize2"
         end
       end
-      if retval.nil? && num.class == Fixnum
+      if retval.nil? && num.class == Integer
         retval = "keycode#{num}"
       end
       retval
     end
+
+    UTF_8_2_BYTE_BEGIN = 0xc2
+    UTF_8_2_BYTE_END = 0xdf
+    UTF_8_3_BYTE_BEGIN = 0xe0
+    UTF_8_3_BYTE_END = 0xef
+    UTF_8_4_BYTE_BEGIN = 0xf0
+    UTF_8_4_BYTE_END = 0xf4
   end
 
   class Diakonos
@@ -242,40 +240,104 @@ module Diakonos
       char > 31 && char < 255 && char != BACKSPACE
     end
 
+    # @param [Integer] c The first byte of a UTF-8 byte sequence
+    # @return [String] nil if c is not the beginning of a multi-byte sequence
+    def utf_8_bytes_to_char(c, mode)
+      if Keying::UTF_8_2_BYTE_BEGIN <= c && c <= Keying::UTF_8_2_BYTE_END
+        # 2-byte character
+        byte_array = [c, @modes[mode].window.getch.ord]
+      elsif Keying::UTF_8_3_BYTE_BEGIN <= c && c <= Keying::UTF_8_3_BYTE_END
+        # 3-byte character
+        byte_array = [
+          c,
+          @modes[mode].window.getch.ord,
+          @modes[mode].window.getch.ord,
+        ]
+      elsif Keying::UTF_8_4_BYTE_BEGIN <= c && c <= Keying::UTF_8_4_BYTE_END
+        # 4-byte character
+        byte_array = [
+          c,
+          @modes[mode].window.getch.ord,
+          @modes[mode].window.getch.ord,
+          @modes[mode].window.getch.ord,
+        ]
+      else
+        return nil
+      end
+
+      byte_array.pack('C*').force_encoding('utf-8')
+    end
+
+    # @param [Integer] c The ordinal (number) of a character
+    # @param [String] mode
+    # @return [Boolean] true iff c began a UTF-8 byte sequence
+    def handle_utf_8(c, mode)
+      utf_8_char = utf_8_bytes_to_char(c, mode)
+      if utf_8_char
+        self.type_character utf_8_char, mode
+        true
+      end
+    end
+
+    # Handle paste from a GUI (like x.org).  i.e. Shift-Insert
+    def handle_gui_paste(mode)
+      s = ""
+      ch = nil
+
+      loop do
+        ch = nil
+        begin
+          Timeout::timeout(0.02) do
+            ch = @modes[mode].window.getch
+          end
+        rescue Timeout::Error => e
+          break
+        end
+        break  if ch.nil?
+
+        c = ch.ord
+        utf_8_char = self.utf_8_bytes_to_char(c, mode)
+
+        if utf_8_char
+          s << utf_8_char
+        elsif self.typeable?(c)
+          s << c
+        elsif c == ENTER && mode == 'edit'
+          s << "\n"
+        else
+          break
+        end
+      end
+
+      if ! s.empty?
+        case mode
+        when 'edit'
+          buffer_current.paste s, Buffer::TYPING
+        when 'input'
+          @readline.paste s
+        end
+      end
+
+      if ch
+        process_keystroke( [], mode, ch )
+      end
+    end
+
     # context is an array of characters (bytes) which are keystrokes previously
     # typed (in a chain of keystrokes)
     def process_keystroke( context = [], mode = 'edit', ch = nil )
       ch ||= @modes[ mode ].window.getch
       return  if ch.nil?
-      c = ch.ord
 
-      # UTF-8
-      if 194 <= c && c <= 244
-        if 194 <= c && c <= 223
-          # 2-byte character
-          byte_array = [c, @modes[mode].window.getch.ord]
-        elsif 224 <= c && c <= 239
-          # 3-byte character
-          byte_array = [
-            c,
-            @modes[mode].window.getch.ord,
-            @modes[mode].window.getch.ord,
-          ]
-        elsif 240 <= c && c <= 244
-          # 4-byte character
-          byte_array = [
-            c,
-            @modes[mode].window.getch.ord,
-            @modes[mode].window.getch.ord,
-            @modes[mode].window.getch.ord,
-          ]
-        end
-
-        char = byte_array.pack('C*').force_encoding('utf-8')
-        type_character char, mode
+      if ch == Curses::KEY_MOUSE
+        handle_mouse_event
         return
       end
 
+      c = ch.ord
+
+      self.handle_utf_8(c, mode) and return
+
       if @capturing_keychain
         capture_keychain c, context
       elsif @capturing_mapping
@@ -283,70 +345,11 @@ module Diakonos
       else
 
         if context.empty? && typeable?( c )
-          type_character ch, mode
-
-          # Handle X windows paste
-          s = ""
-          loop do
-            ch = nil
-            begin
-              Timeout::timeout( 0.02 ) do
-                ch = @modes[ mode ].window.getch
-              end
-            rescue Timeout::Error => e
-              break
-            end
-            break  if ch.nil?
-
-            c = ch.ord
-            # UTF-8
-            if 194 <= c && c <= 244
-              if 194 <= c && c <= 223
-                # 2-byte character
-                byte_array = [c, @modes[mode].window.getch.ord]
-              elsif 224 <= c && c <= 239
-                # 3-byte character
-                byte_array = [
-                  c,
-                  @modes[mode].window.getch.ord,
-                  @modes[mode].window.getch.ord,
-                ]
-              elsif 240 <= c && c <= 244
-                # 4-byte character
-                byte_array = [
-                  c,
-                  @modes[mode].window.getch.ord,
-                  @modes[mode].window.getch.ord,
-                  @modes[mode].window.getch.ord,
-                ]
-              end
-
-              char = byte_array.pack('C*').force_encoding('utf-8')
-              s << char
-            elsif typeable?( c )
-              s << c
-            elsif c == ENTER && mode == 'edit'
-              s << "\n"
-            else
-              break
-            end
-          end
-
-          if ! s.empty?
-            case mode
-            when 'edit'
-              buffer_current.paste s, Buffer::TYPING
-            when 'input'
-              @readline.paste s
-            end
-          end
-
-          if ch
-            process_keystroke( [], mode, ch )
-          end
-
+          self.type_character ch, mode
+          self.handle_gui_paste(mode)
           return
         end
+
         keychain_pressed = context.concat [ c ]
 
         function_and_args = (
@@ -374,7 +377,7 @@ module Diakonos
 
           begin
             if buffer_current.search_area? && ! ( /^(?:find|readline)/ === to_eval )
-              buffer_current.search_area = nil
+              buffer_current.clear_search_area
             end
             eval to_eval, nil, "eval"
             @functions_last << to_eval  unless to_eval == "repeat_last"
@@ -402,7 +405,7 @@ module Diakonos
 
     def type_character( c, mode = 'edit' )
       if @macro_history
-        @macro_history.push "type_character #{c}, #{mode.inspect}"
+        @macro_history.push "type_character #{c.inspect}, #{mode.inspect}"
       end
       @there_was_non_movement = true
 
@@ -411,6 +414,9 @@ module Diakonos
         buffer_current.delete_selection Buffer::DONT_DISPLAY
         buffer_current.insert_string c
         cursor_right Buffer::STILL_TYPING
+        if c =~ @indent_triggers[buffer_current.language]
+          buffer_current.parsed_indent cursor_eol: true
+        end
       when 'input'
         if ! @readline.numbered_list?
           @readline.paste c
