@@ -7,8 +7,8 @@ module Diakonos
       @list_buffer = open_file( @list_filename )
     end
 
-    def close_list_buffer
-      close_buffer @list_buffer
+    def close_list_buffer( opts = {} )
+      close_buffer  @list_buffer, opts
       @list_buffer = nil
     end
 
@@ -17,7 +17,7 @@ module Diakonos
     end
 
     def list_item_selected?
-      @list_buffer and @list_buffer.selecting?
+      @list_buffer && @list_buffer.selecting?
     end
 
     def current_list_item
