@@ -37,7 +37,7 @@ def doCommand( command )
 end
 
 def printUsage
-    puts "#{$0} <version number> [--work-dir <dir>] [--step]"
+    puts "#{$0} <version number> [--work-dir <dir>] [--step] [-p <scp port number>]"
 end
 
 # ---------------
@@ -49,6 +49,7 @@ end
 
 version = nil
 work_dir = '/misc/pistos/unpack'
+scp_port = 22
 $step = false
 
 args = ARGV.dup
@@ -58,6 +59,8 @@ while args.length > 0
         when '-h', '--help'
             printUsage
             exit 1
+        when '-p', '-P', '--port'
+          scp_port = args.shift
         when '--step'
             $step = true
         when '--work-dir'
@@ -68,7 +71,7 @@ while args.length > 0
 end
 
 puts "git tag and export..."
-doCommand "git tag -a v#{version} -m 'Tagged Diakonos version #{version}.'"
+doCommand "git co v#{version} || git tag -a v#{version} -m 'Tagged Diakonos version #{version}.'"
 doCommand "git archive --format=tar --prefix=diakonos-#{version}/ refs/tags/v#{version} | bzip2 > diakonos-#{version}.tar.bz2"
 doCommand "git archive --format=tar --prefix=diakonos-#{version}/ refs/tags/v#{version} | gzip > diakonos-#{version}.tar.gz"
 
@@ -77,13 +80,12 @@ doCommand( "md5sum diakonos-#{version}.tar.gz" )
 doCommand( "md5sum diakonos-#{version}.tar.bz2" )
 
 puts "Copying files to website..."
-doCommand( "scp diakonos-#{version}.tar.bz2 diakonos-#{version}.tar.gz CHANGELOG README.rdoc pistos@diakonos.pist0s.ca:/var/www/diakonos.pist0s.ca/archives" )
+doCommand( "scp -P #{scp_port} diakonos-#{version}.tar.bz2 diakonos-#{version}.tar.gz CHANGELOG README.rdoc pistos@diakonos.pist0s.ca:/var/www/diakonos.pist0s.ca/archives" )
 
 puts "Release complete."
 puts
-puts "Announcement sites:"
-puts "1) rubyforge.org"
-puts "4) diakonos.pist0s.ca"
-puts "5) blog.purepistos.net"
-puts "6) RAA"
-puts "7) http://en.wikipedia.org/wiki/Diakonos"
+puts "Release/announce on sites:"
+puts "1) diakonos.pist0s.ca"
+puts "2) rubyforge.org"
+puts "3) RAA"
+puts "4) http://en.wikipedia.org/wiki/Diakonos"
